/* JNI Dazuko Interface Wrapper. Interace with Dazuko for file access control.
   Written by Alex Ellwein <aellwein@antivir.de>

Copyright (C) 2003, H+BEDV Datentechnik GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of Dazuko nor the names of its contributors may be used
to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include <signal.h>				
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include "dazukoio.h"

#include <jni.h>
#include "org_dazuko_Dazuko.h"

jint JNICALL Java_org_dazuko_Dazuko_register(JNIEnv* env, jobject o, jstring s)
{
	int ret;
	const char* str = (*env)->GetStringUTFChars(env, s, 0);

  #ifdef _WRAPPER_DEBUG_
	printf("wrapper: register with Dazuko as \"%s\"\n", str); 
  #endif

	ret = dazukoRegister(str);
  	
	(*env)->ReleaseStringUTFChars(env, s, str);

	return ret;
}

jint JNICALL Java_org_dazuko_Dazuko_unregister(JNIEnv* env, jobject o)
{
#ifdef _WRAPPER_DEBUG_
	printf("wrapper: unregistering by Dazuko\n");
#endif

	return dazukoUnregister();
}

jint JNICALL Java_org_dazuko_Dazuko_setAccessMask(JNIEnv* env, jobject o, jint mask)
{
#ifdef _WRAPPER_DEBUG_
	printf("wrapper: setting access mask\n");
#endif

	return dazukoSetAccessMask(mask);
}

jint JNICALL Java_org_dazuko_Dazuko_addIncludePath(JNIEnv* env, jobject o, jstring s)
{
	int ret;
	const char* str = (*env)->GetStringUTFChars(env, s, 0);

#ifdef _WRAPPER_DEBUG_
      printf("wrapper: adding include path: \"%s\"\n", str);
#endif

	ret = dazukoAddIncludePath(str);

	(*env)->ReleaseStringUTFChars(env, s, str);

	return ret;
}

jint JNICALL Java_org_dazuko_Dazuko_addExcludePath(JNIEnv* env, jobject o, jstring s)
{
	int ret;
	const char* str = (*env)->GetStringUTFChars(env, s, 0);

#ifdef _WRAPPER_DEBUG_
   	printf("wrapper: adding exclude path: \"%s\"\n", str);	
#endif
	ret = dazukoAddExcludePath(str);

	(*env)->ReleaseStringUTFChars(env, s, str);

	return ret;
}

jint JNICALL Java_org_dazuko_Dazuko_removeAllPaths(JNIEnv* env, jobject o)
{
#ifdef _WRAPPER_DEBUG_
	printf("removing all paths\n");
#endif

	return dazukoRemoveAllPaths();
}

jobject JNICALL Java_org_dazuko_Dazuko_getAccess(JNIEnv* env, jobject o, jobject access)
{
	int ret;
	struct access_t acc;
	jclass cls;
	jfieldID fid;
	jstring jstr;
	jobject result;
	
	ret = dazukoGetAccess(&acc);

	if (ret == 0)
	{
#ifdef _WRAPPER_DEBUG_
	  	printf("wrapper: got access to file from dazuko \n");
#endif
	  
		/* Now we need to move all structure data to Java class */
		
		cls = (*env)->GetObjectClass(env, access);				/* getting class type from java */
		result = (*env)->AllocObject(env, (*env)->GetObjectClass(env, access)); /* allocate same object type as access object */
		if (cls == 0) return 0;
		fid = (*env)->GetFieldID(env, cls, "event", "I");
		if (fid == 0) return 0;
		(*env)->SetIntField(env, result, fid, acc.event);
		fid = (*env)->GetFieldID(env, cls, "flags", "I");
		if (fid == 0) return 0;
		(*env)->SetIntField(env, result, fid, acc.o_flags);
		fid = (*env)->GetFieldID(env, cls, "mode", "I");
		if (fid == 0) return 0;
		(*env)->SetIntField(env, result, fid, acc.o_mode);
		fid = (*env)->GetFieldID(env, cls, "uid", "I");
		if (fid == 0) return 0;
                (*env)->SetIntField(env, result, fid, acc.uid);
		fid = (*env)->GetFieldID(env, cls, "pid", "I");
		if (fid == 0) return 0;
		(*env)->SetIntField(env, result, fid, acc.pid);
		fid = (*env)->GetFieldID(env, cls, "filename", "Ljava/lang/String;");
		if (fid == 0) return 0;
		jstr = (*env)->NewStringUTF(env, acc.filename);
		(*env)->SetObjectField(env, result, fid, jstr);
		
		/* for now, all of the access object member values are filled. */
		return result;								/* return access object */
	}

	/* something goes wrong, return null reference */
#ifdef _WRAPPER_DEBUG_
	printf("wrapper: failed to get access from dazuko\n");
#endif

	return NULL;
}

jint JNICALL Java_org_dazuko_Dazuko_returnAccess(JNIEnv* env, jobject o, jobject access)
{
	struct access_t acc;
	jclass cls;
	jfieldID fid;
	jstring jstr;
	const char* str;
	
#ifdef _WRAPPER_DEBUG_
	printf("wrapper: returning access\n");
#endif

	/* so we need to get data back from Java object into our access structure */
	cls = (*env)->GetObjectClass(env, access);					/* get class type info */
	if (cls == 0) return JNI_FALSE;							/* invalid class       */
        
	fid = (*env)->GetFieldID(env, cls, "deny", "Z");
        if (fid == 0) return 0;
        acc.deny = (*env)->GetBooleanField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "event", "I");
        if (fid == 0) return 0;
        acc.event = (*env)->GetIntField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "flags", "I");
        if (fid == 0) return 0;
        acc.o_flags = (*env)->GetIntField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "mode", "I");
        if (fid == 0) return 0;
        acc.o_mode = (*env)->GetIntField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "uid", "I");
        if (fid == 0) return 0;
        acc.uid = (*env)->GetIntField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "pid", "I");
        if (fid == 0) return 0;
        acc.pid = (*env)->GetIntField(env, access, fid);
        
	fid = (*env)->GetFieldID(env, cls, "filename", "Ljava/lang/String;");
        if (fid == 0) return 0;
	
	jstr = (jstring)(*env)->GetObjectField(env, access, fid);			/* string cast from object */
	str = (*env)->GetStringUTFChars(env, jstr, 0);					/* getting chars from java string */
	strcpy(acc.filename, str);
	(*env)->ReleaseStringUTFChars(env, jstr, str);					/* release java string */
	
	return dazukoReturnAccess(&acc);
}
