/* Example Java program demonstrating the capabilities/interface of Dazuko.
   Written by Alex Ellwein <aellwein@antivir.de>

Copyright (C) 2003, H+BEDV Datentechnik GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of Dazuko nor the names of its contributors may be used
to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

import org.dazuko.*;

/** Simple example program showing how to use
 * Dazuko Java wrapper.
 * 
 * @author Alexander Ellwein <aellwein@antivir.de>
 * @version tested with Dazuko 1.2.0
 *
 */
public class Example
{
	public static void printAccess(DazukoAccess access)
	{
		/* print access data */

		switch (access.getEvent())
		{
			case DazukoAccess.ON_OPEN:
				System.out.print("OPEN ");
				break;
			case DazukoAccess.ON_CLOSE:
				System.out.print("CLOSE ");
				break;
			case DazukoAccess.ON_CLOSE_MODIFIED:
				System.out.print("CLOSE (modified) ");
				break;
			case DazukoAccess.ON_EXEC:
				System.out.print("EXEC ");
				break;
			default:
				System.out.print("???? ");
				break;
		}
		System.out.print("flags: " + access.getFlags() + " ");
		System.out.print("mode: " + access.getMode() + " ");
		System.out.print("uid: " + access.getUID() + " ");
		System.out.print("pid: " + access.getPID() + " ");
		System.out.print("file: " + access.getFilename());
		System.out.println("");
	}

	/** programm entry point 
	 * 
	 * @param args parameters (none needed here)
	 */
	public static void main(String[] args)
	{
		Dazuko dazuko;					/* our wrapper */
		DazukoAccess access = new DazukoAccess();	/* access structure */
		dazuko = new Dazuko();				/* build wrapper */
		boolean RUNNING = true;
		
		if (dazuko.register("DAZUKO_EXAMPLE_JAVA") != 0)
		{
			System.out.println("error: failed to register with Dazuko");
			System.exit(-1);
		}

		System.out.println("registered with Dazuko successfully");

		if (dazuko.setAccessMask(DazukoAccess.ON_OPEN | 
			DazukoAccess.ON_CLOSE |
			DazukoAccess.ON_CLOSE_MODIFIED |
			DazukoAccess.ON_EXEC) != 0)
		{
			System.out.println("error: failed to set access mask");
			dazuko.unregister();
			System.exit(-1);
		}

		System.out.println("set access mask successfully");   		

		/* set scan path */
		if (dazuko.addIncludePath("/home/") != 0)
		{
			System.out.println("failed to add include path");
			dazuko.unregister();
			System.exit(-1);
		}

		System.out.println("set scan path successfully");

		while(RUNNING)
		{
			/* get an access */
			access = dazuko.getAccess(access);

			if (access != null)
			{
				printAccess(access);

				/* always allow access */
				access.setDeny(false);

				/* return access (IMPORTANT, the kernel is waiting for us!) */

				if (dazuko.returnAccess(access) != 0)
				{
					System.out.println("error: failed to return access");
					RUNNING = false;
				}
			}
			else
			{
				System.out.println("warning: failed to get an access");
				RUNNING = false;
			}
		}

		/* unregister with Dazuko */
		dazuko.unregister();
	}
}
