/* Java Dazuko Interface. Interace with Dazuko for file access control.
   Written by Alex Ellwein <aellwein@antivir.de>

Copyright (C) 2003, H+BEDV Datentechnik GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of Dazuko nor the names of its contributors may be used
to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package org.dazuko;

/** This class contains a bunch of functions 
 * allowing to pass access control structure 
 * (<code>DazukoAccess</code>) to or read it from Dazuko. 
 *  
 * @see org.dazuko.DazukoAccess
 * @author Alexander Ellwein <ellwein@gmx.net>
 * @version tested with Dazuko 1.1.2
 *
 */
public final class Dazuko
{
	/** Default constructor
	 *
	 * @see java.lang.Object#Object()
	 */
	public Dazuko()
	{
		super();	// object's constructor
		System.loadLibrary("DazukoWrapper");		// loads wrapper
	}

	/** Registers the handler with specified name.
	 * @return true. if registration was successful.
	 */	
	public final native int register(String groupName);	
	
	/** Unregisters the handler.
	 */
	public final native int unregister();
	
	/** Sets the access mask.
	 * @return true, if the access mask was set successfully.
	 */
	public final native int setAccessMask(int accessMask);
	
	/** Adds a path that should be included by Dazuko.
	 * @return true, if the path was added successfully.
	 */
	public final native int addIncludePath(String path);
	
	/** Adds a path that should be excluded by Dazuko.
	 * @return true, if the path was added successfully. 
	 */
	public final native int addExcludePath(String path);
	
	/** Removes all paths from the path's list.
	 * @return true, if the paths were removed successfully.
	 */
	public final native int removeAllPaths();

	/** Gets an access to Dazuko interface using DazukoAccess class
	 * container.<br>
	 * <b>IMPORTANT:</b> latency time between <code>getAccess</code> and
	 * <code>returnAccess</code> should be SMALL !
	 * <br>
	 * @param access Access container.
	 * @return DazukoAccess modified container.
	 */
	public final native DazukoAccess getAccess(DazukoAccess access);
	
	/** Returns an access container to Dazuko.<br>
	 * <b>IMPORTANT:</b> latency time between <code>getAccess</code>
	 *  and <code>returnAccess</code> should be SMALL !<br>
	 * @param access Access container.
	 * @return boolean true, if access container was accepted by Dazuko.
	 */
	public final native int returnAccess(DazukoAccess access);
	
}
