/* Java Dazuko Access. Access structure for Dazuko file access control.
   Written by Alex Ellwein <aellwein@antivir.de>

Copyright (C) 2003, H+BEDV Datentechnik GmbH
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of Dazuko nor the names of its contributors may be used
to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package org.dazuko;

/** This container class can be passed to Dazuko  
 * in order to control access on files accessed
 * by other applications.
 * 
 * @author Alexander Ellwein <aellwein@antivir.de>
 * @version tested with Dazuko 1.2.0
 *
 */
public final class DazukoAccess
{
	private boolean deny = false;
	private int event = 0;
	private int flags = 0;
	private int mode = 0;
	private int uid = 0;
	private	int pid	= 0;
	private String filename = null;
	
	/** open file mode
	 * 
	 */
	public final static int ON_OPEN		= 1;
	
	/** close file mode
	 * 
	 */
	public final static int ON_CLOSE	= 2;
	
	/** execute file mode
	 * 
	 */
	public final static int ON_EXEC		= 4;
	
	/** close modified file mode 
	 *
	 */
	public final static int ON_CLOSE_MODIFIED = 8;
	
	/** Default constructor 
	 * 
	 * @see java.lang.Object#Object()
	 */
	public DazukoAccess()
	{
		deny = false;
		event = 0;
		flags = 0;
		mode = 0;
		uid = 0;
		filename = null;
	}
	
	/** If access is denied, user will not be able
	 * to do something on that file.
	 * @param deny if <code>true</code>, this access will be denied
	 */ 					
	public void setDeny(boolean deny)
	{
		this.deny = deny;
	}
	
	/** Returns access type (ON_OPEN, etc)
	 * 
	 * @return int event type
	 */
	public final int getEvent()
	{
		return event;
	}
	
	/** Returns access flags.
	 * 
	 * @return int access flags
	 */
	public final int getFlags()
	{
		return flags;
	}
	
	/** Returns application's file open mode
	 * 
	 * @return int application's open mode
	 */  
	public final int getMode()
	{
		return mode;
	}
	
	/** Returns User ID (UID) of user who attempted 
	 * to access file.
	 * 
	 * @return int User ID (UID)
	 */
	public final int getUID()
	{
		return uid;	
	}
	
	/** Returns process id (PID) of program that 
	 * attempted to access file.
	 * 
	 * @return int process id
	 */
	public final int getPID()
	{
		return pid;
	}

	/** Returns full filename of accessed file.
	 * @return String filename of accessed file.
	 */	
	public final String getFilename()
	{
		return filename;
	}
}
